from Helper import Helper
import urllib2, urllib
import StringIO
import gzip
import HTMLParser
# from qaException.apiQaException import ApiQaException

class HttpHelper(Helper):
	"""
	Helper class for sending requests of HTTP with GET and POST
	"""


	login_cred = {} 		#: Auth for HTTP access. type(dict) : {'username':'x','password':'y'} 
	client_cred = {} 		#: Auth for client credentials.  to be added to header. 
	api_url = None 			#: 
	api_param = None
	request_type = None
	#logPath = None
	responseFormat = None
	

	def __init__(self,request_type,api_url,login_cred={},client_cred={},request_header={}):
		"""
		Args :
		request_type (str) : Takes POST or GET as type of HTTP requests
		api_url (str) : The url to the base of the API call (everything but the call itself)
		login_cred (dict) : Optionally takes a username and password if the HTTP takes authentication

		Returns : null

		Raises : null
		"""

		Helper.__init__(self)
		self.request_type = request_type
		self.api_url = api_url
		self.login_cred = login_cred
		self.client_cred = client_cred
		self.request_header = request_header

	def getResponse(self,constructed_url):
		"""
		Sends the api call and returns the response

		Args : 
		constructed_url: (str) url and parameters to send

		Returns : 
		the_page: (json) response body from the api call
		code: (int) response code from the HTTP response
		error: (string) error response if api call returns error

		Raises : urllib2.HTTPError,urllib2.URLError
		"""

		# Initialize returned variables
		the_page = None
		code = None
		error = None

		# add authentication if you need it
		if len(self.login_cred) >= 2:
			self.getAuth(constructed_url)

		try:
			self.logger.debug("The Url targeted is : %s"%constructed_url)
			# Send the API call
			if len(self.request_header) >= 1:
				constructed_url = urllib2.Request(constructed_url, None, self.request_header)
			pagehandle = urllib2.urlopen(constructed_url)

			# Get the response status code
			code = pagehandle.getcode()
			# Read the response from the request
			the_page = pagehandle.read()
			if pagehandle.info().get('Content-Encoding') == 'gzip':
				buf = StringIO.StringIO(the_page)
				f = gzip.GzipFile(fileobj=buf)
				the_page = f.read()
			

		except urllib2.HTTPError, e:
		
			# Get the error code and message to send back if something went wrong
			#raise ApiQaException(e)
			code = e.code
			error = e.read()
		except urllib2.URLError, e:
			#if we get a timeout
			#code = e.code
			error = e.message
			
		return the_page, code, error

	def getHTTPResponse(self,call,data):
		"""
		constructs and sends the api call and returns the response

		Args : 
		call: (str) Name of the api service you are calling
		data: (list) The parameters you will be providing the api service

		Returns : null

		Raises : null
		"""
		


		# add the API call to the URL
		theurl = self.api_url+call

		if self.request_type == "GET":
			constructed_url = self.createUrl(data,theurl)
			self.logger.debug("Sending a GET HTTP request")
		elif self.request_type == "POST":
			self.logger.debug("Sending a POST HTTP request")
			# If credentials need to be sent in the header
			if len(self.client_cred) >=2:
				headers = {"Authorization":self.client_cred['key']+":"+self.client_cred['secretKey']}
				print headers
				constructed_url = urllib2.Request(theurl,urllib.urlencode(data),headers)
			else:
				constructed_url = urllib2.Request(theurl,urllib.urlencode(data))

		return self.getResponse(constructed_url)




	def createUrl(self,data,theurl):
		"""
		Helper function to create url in either path or parameter api methods 

		Args :
		data: (str) values sent to api query
		theurl: (str) the http path to the api method

		Returns : null

		Raises : null
		"""
		
		#path args type
		if isinstance(data,str):
			self.logger.debug("We are using path arguments for the url parameters")
			return theurl+'/'+data
		#query args type
		elif isinstance(data,dict):
			url_values = urllib.urlencode(data)
			self.logger.debug("We are using query arguments for the url parameters")
			return theurl+'?'+url_values


	def getAuth(self,theurl):
		"""
		Creates the authentication for accessing api

		Args :
		theurl: (str) the http path to the api method

		Returns : null

		Raises :null
		"""

		# create a handler for the username / password
		passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
		# none is so that all pages under the realm are opened with password
		passman.add_password(None, theurl, self.login_cred['username'], self.login_cred['password'])

		# authenticate and open the website
		authhandler = urllib2.HTTPBasicAuthHandler(passman)
		opener = urllib2.build_opener(authhandler)
		urllib2.install_opener(opener)
		self.logger.debug("We are using authentication")


	def decodeHtmlString(self,string):
		"""
		translates html entities into a readable string

		Args :
		string: (str) an html entity

		Returns : 
		unescaped: (str) returns a readable string

		Raises :null
		"""

		h = HTMLParser.HTMLParser()
		return h.unescape(string)
